@@ -6,7 +6,7 @@ package Excel::Writer::XLSX::Package::Packager;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2010, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2011, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -27,7 +27,7 @@ use Excel::Writer::XLSX::Package::Styles;
 use Excel::Writer::XLSX::Package::Theme;
 
 our @ISA     = qw(Exporter);
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 
 ###############################################################################
@@ -48,11 +48,14 @@ sub new {
     my $class = shift;
 
     my $self = {
-        _package_dir  => '',
-        _workbook     => undef,
-        _sheet_names  => [],
-        _sheet_count  => 0,
-        _named_ranges => [],
+        _package_dir      => '',
+        _workbook         => undef,
+        _sheet_names      => [],
+        _worksheet_count  => 0,
+        _chartsheet_count => 0,
+        _chart_count      => 0,
+        _drawing_count    => 0,
+        _named_ranges     => [],
     };
 
 
@@ -88,10 +91,20 @@ sub _add_workbook {
     my $workbook    = shift;
     my @sheet_names = @{ $workbook->{_sheetnames} };
 
-    $self->{_workbook}     = $workbook;
-    $self->{_sheet_names}  = \@sheet_names;
-    $self->{_sheet_count}  = scalar @sheet_names;
-    $self->{_named_ranges} = $workbook->{_named_ranges};
+    $self->{_workbook}      = $workbook;
+    $self->{_sheet_names}   = \@sheet_names;
+    $self->{_chart_count}   = scalar @{ $workbook->{_charts} };
+    $self->{_drawing_count} = scalar @{ $workbook->{_drawings} };
+    $self->{_named_ranges}  = $workbook->{_named_ranges};
+
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        if ( $worksheet->{_is_chartsheet} ) {
+            $self->{_chartsheet_count}++;
+        }
+        else {
+            $self->{_worksheet_count}++;
+        }
+    }
 }
 
 
@@ -107,6 +120,9 @@ sub _create_package {
 
     $self->_write_workbook_file();
     $self->_write_worksheet_files();
+    $self->_write_chartsheet_files();
+    $self->_write_chart_files();
+    $self->_write_drawing_files();
     $self->_write_shared_strings_file();
     $self->_write_app_file();
     $self->_write_core_file();
@@ -116,6 +132,8 @@ sub _create_package {
     $self->_write_root_rels_file();
     $self->_write_workbook_rels_file();
     $self->_write_worksheet_rels_files();
+    $self->_write_chartsheet_rels_files();
+    $self->_write_drawing_rels_files();
 }
 
 
@@ -154,12 +172,91 @@ sub _write_worksheet_files {
 
     my $index = 1;
     for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        next if $worksheet->{_is_chartsheet};
+
         $worksheet->_set_xml_writer(
             $dir . '/xl/worksheets/sheet' . $index++ . '.xml' );
         $worksheet->_assemble_xml_file();
 
     }
+}
 
+
+###############################################################################
+#
+# _write_chartsheet_files()
+#
+# Write the chartsheet files.
+#
+sub _write_chartsheet_files {
+
+    my $self = shift;
+    my $dir  = $self->{_package_dir};
+
+    mkdir $dir . '/xl';
+    mkdir $dir . '/xl/chartsheets';
+
+    my $index = 1;
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        next unless $worksheet->{_is_chartsheet};
+
+        $worksheet->_set_xml_writer(
+            $dir . '/xl/chartsheets/sheet' . $index++ . '.xml' );
+        $worksheet->_assemble_xml_file();
+
+    }
+}
+
+
+###############################################################################
+#
+# _write_chart_files()
+#
+# Write the chart files.
+#
+sub _write_chart_files {
+
+    my $self = shift;
+    my $dir  = $self->{_package_dir};
+
+    return unless @{ $self->{_workbook}->{_charts} };
+
+    mkdir $dir . '/xl';
+    mkdir $dir . '/xl/charts';
+
+    my $index = 1;
+    for my $chart ( @{ $self->{_workbook}->{_charts} } ) {
+        $chart->_set_xml_writer(
+            $dir . '/xl/charts/chart' . $index++ . '.xml' );
+        $chart->_assemble_xml_file();
+
+    }
+}
+
+
+###############################################################################
+#
+# _write_drawing_files()
+#
+# Write the drawing files.
+#
+sub _write_drawing_files {
+
+    my $self = shift;
+    my $dir  = $self->{_package_dir};
+
+    return unless $self->{_drawing_count};
+
+    mkdir $dir . '/xl';
+    mkdir $dir . '/xl/drawings';
+
+    my $index = 1;
+    for my $drawing ( @{ $self->{_workbook}->{_drawings} } ) {
+        $drawing->_set_xml_writer(
+            $dir . '/xl/drawings/drawing' . $index++ . '.xml' );
+        $drawing->_assemble_xml_file();
+
+    }
 }
 
 
@@ -207,13 +304,22 @@ sub _write_app_file {
     mkdir $dir . '/docProps';
 
     # Add the Worksheet heading pairs.
-    $app->_add_heading_pair( [ 'Worksheets', $self->{_sheet_count} ] );
+    $app->_add_heading_pair( [ 'Worksheets', $self->{_worksheet_count} ] );
+
+    # Add the Chartsheet heading pairs.
+    $app->_add_heading_pair( [ 'Charts', $self->{_chartsheet_count} ] );
 
     # Add the Worksheet parts.
-    for my $sheet_name ( @{ $self->{_sheet_names} } ) {
-        $app->_add_part_name( $sheet_name );
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        next if $worksheet->{_is_chartsheet};
+        $app->_add_part_name( $worksheet->get_name() );
     }
 
+    # Add the Chartsheet parts.
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        next unless $worksheet->{_is_chartsheet};
+        $app->_add_part_name( $worksheet->get_name() );
+    }
 
     # Add the Named Range heading pairs.
     if ( my $range_count = scalar @{ $self->{_named_ranges} } ) {
@@ -266,8 +372,23 @@ sub _write_content_types_file {
     my $dir     = $self->{_package_dir};
     my $content = Excel::Writer::XLSX::Package::ContentTypes->new();
 
-    for my $i ( 1 .. @{ $self->{_sheet_names} } ) {
-        $content->_add_sheet_name( 'sheet' . $i );
+    my $worksheet_index  = 1;
+    my $chartsheet_index = 1;
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        if ( $worksheet->{_is_chartsheet} ) {
+            $content->_add_chartsheet_name( 'sheet' . $chartsheet_index++ );
+        }
+        else {
+            $content->_add_worksheet_name( 'sheet' . $worksheet_index++ );
+        }
+    }
+
+    for my $i ( 1 .. $self->{_chart_count} ) {
+        $content->_add_chart_name( 'chart' . $i );
+    }
+
+    for my $i ( 1 .. $self->{_drawing_count} ) {
+        $content->_add_drawing_name( 'drawing' . $i );
     }
 
     # Add the sharedString rel if there is string data in the workbook.
@@ -377,9 +498,18 @@ sub _write_workbook_rels_file {
     mkdir $dir . '/xl';
     mkdir $dir . '/xl/_rels';
 
-    for my $index ( 1 .. $self->{_sheet_count} ) {
-        $rels->_add_document_relationship( '/worksheet',
-            'worksheets/sheet' . $index );
+    my $worksheet_index  = 1;
+    my $chartsheet_index = 1;
+
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        if ( $worksheet->{_is_chartsheet} ) {
+            $rels->_add_document_relationship( '/chartsheet',
+                'chartsheets/sheet' . $chartsheet_index++ );
+        }
+        else {
+            $rels->_add_document_relationship( '/worksheet',
+                'worksheets/sheet' . $worksheet_index++ );
+        }
     }
 
     $rels->_add_document_relationship( '/theme',  'theme/theme1' );
@@ -400,7 +530,7 @@ sub _write_workbook_rels_file {
 # _write_worksheet_rels_files()
 #
 # Write the worksheet .rels files for worksheets that contain links to external
-# data such as hyperlinks.
+# data such as hyperlinks or drawings.
 #
 sub _write_worksheet_rels_files {
 
@@ -411,8 +541,15 @@ sub _write_worksheet_rels_files {
 
     my $index = 0;
     for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+
+        next if $worksheet->{_is_chartsheet};
+
         $index++;
-        next unless $worksheet->{_hlink_count};
+
+        my @external_links = (@{ $worksheet->{_external_hlinks} },
+          @{ $worksheet->{_external_dlinks} });
+
+        next unless @external_links;
 
         # Create the worksheet .rels dir if required.
         if ( !$existing_rels_dir ) {
@@ -424,7 +561,7 @@ sub _write_worksheet_rels_files {
 
         my $rels = Excel::Writer::XLSX::Package::Relationships->new();
 
-        for my $link_data ( @{ $worksheet->{_external_links} } ) {
+        for my $link_data ( @external_links ) {
             $rels->_add_worksheet_relationship( @$link_data );
         }
 
@@ -436,6 +573,92 @@ sub _write_worksheet_rels_files {
 }
 
 
+###############################################################################
+#
+# _write_chartsheet_rels_files()
+#
+# Write the chartsheet .rels files for links to drawing files.
+#
+sub _write_chartsheet_rels_files {
+
+    my $self              = shift;
+    my $dir               = $self->{_package_dir};
+    my $existing_rels_dir = 0;
+
+
+    my $index = 0;
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+
+        next unless $worksheet->{_is_chartsheet};
+
+        $index++;
+
+        my @external_links = @{ $worksheet->{_external_dlinks} };
+
+        next unless @external_links;
+
+        # Create the chartsheet .rels dir if required.
+        if ( !$existing_rels_dir ) {
+            mkdir $dir . '/xl';
+            mkdir $dir . '/xl/chartsheets';
+            mkdir $dir . '/xl/chartsheets/_rels';
+            $existing_rels_dir = 1;
+        }
+
+        my $rels = Excel::Writer::XLSX::Package::Relationships->new();
+
+        for my $link_data ( @external_links ) {
+            $rels->_add_worksheet_relationship( @$link_data );
+        }
+
+        # Create the .rels file such as /xl/chartsheets/_rels/sheet1.xml.rels.
+        $rels->_set_xml_writer(
+            $dir . '/xl/chartsheets/_rels/sheet' . $index . '.xml.rels' );
+        $rels->_assemble_xml_file();
+    }
+}
+
+
+###############################################################################
+#
+# _write_drawing_rels_files()
+#
+# Write the drawing .rels files for worksheets that contain charts or drawings.
+#
+sub _write_drawing_rels_files {
+
+    my $self              = shift;
+    my $dir               = $self->{_package_dir};
+    my $existing_rels_dir = 0;
+
+
+    my $index = 0;
+    for my $worksheet ( @{ $self->{_workbook}->{_worksheets} } ) {
+        next unless @{ $worksheet->{_drawing_links} };
+        $index++;
+
+        # Create the drawing .rels dir if required.
+        if ( !$existing_rels_dir ) {
+            mkdir $dir . '/xl';
+            mkdir $dir . '/xl/drawings';
+            mkdir $dir . '/xl/drawings/_rels';
+            $existing_rels_dir = 1;
+        }
+
+        my $rels = Excel::Writer::XLSX::Package::Relationships->new();
+
+        for my $drawing_data ( @{ $worksheet->{_drawing_links} } ) {
+            $rels->_add_document_relationship( @$drawing_data );
+        }
+
+        # Create the .rels file such as /xl/drawings/_rels/sheet1.xml.rels.
+        $rels->_set_xml_writer(
+            $dir . '/xl/drawings/_rels/drawing' . $index . '.xml.rels' );
+        $rels->_assemble_xml_file();
+    }
+}
+
+
 ###############################################################################
 #
 # _localtime_to_iso8601_date()
@@ -496,10 +719,10 @@ At its simplest an Excel XLSX file contains the following elements:
     | |____ theme
     | | |____ theme1.xml
     | |
-    | |____ _rels
+    | |_____rels
     |   |____ workbook.xml.rels
     |
-    |____ _rels
+    |_____rels
       |____ .rels
 
 
